@@ -369,6 +369,38 @@ class _Parser(object):
                     ignore_missing_keys=self._ignore_missing_keys,
                 ).parse(cond)
             ]
+        if operator == '$slice':
+            if not isinstance(value, list):
+                raise OperationFailure('$slice only supports a list as its argument')
+            if len(value) < 2 or len(value) > 3:
+                raise OperationFailure('Expression $slice takes at least 2 arguments, and at most '
+                                       '3, but {} were passed in'.format(len(value)))
+            array_value = self.parse(value[0])
+            if not isinstance(array_value, list):
+                raise OperationFailure(
+                    'First argument to $slice must be an array, but is of type: {}'
+                    .format(type(array_value)))
+            for num, v in zip(('Second', 'Third'), value[1:]):
+                if not isinstance(v, six.integer_types):
+                    raise OperationFailure(
+                        '{} argument to $slice must be numeric, but is of type: {}'
+                        .format(num, type(v)))
+            if len(value) > 2 and value[2] <= 0:
+                raise OperationFailure('Third argument to $slice must be '
+                                       'positive: {}'.format(value[2]))
+
+            start = value[1]
+            if start < 0:
+                if len(value) > 2:
+                    stop = len(array_value) + start + value[2]
+                else:
+                    stop = None
+            elif len(value) > 2:
+                stop = start + value[2]
+            else:
+                stop = start
+                start = 0
+            return array_value[start:stop]
 
         raise NotImplementedError(
             "Although '%s' is a valid array operator for the "
@@ -448,9 +480,9 @@ def _accumulate_group(output_fields, group_list):
             if operator in _GROUPING_OPERATOR_MAP:
                 doc_dict[field] = _GROUPING_OPERATOR_MAP[operator](values)
             elif operator == '$first':
-                doc_dict[field] = values[0]
+                doc_dict[field] = values[0] if values else None
             elif operator == '$last':
-                doc_dict[field] = values[-1]
+                doc_dict[field] = values[-1] if values else None
             elif operator == '$addToSet':
                 value = []
                 val_it = (val or None for val in values)
@@ -813,6 +845,9 @@ def _project_by_spec(doc, proj_spec, is_include):
 
         if isinstance(value, dict):
             output[key] = _project_by_spec(value, proj_spec[key], is_include)
+        elif isinstance(value, list):
+            output[key] = [_project_by_spec(array_value, proj_spec[key], is_include)
+                           for array_value in value if isinstance(array_value, dict)]
         elif not is_include:
             output[key] = value
 
@@ -855,10 +890,9 @@ def _handle_project_stage(in_collection, unused_database, options):
             raise OperationFailure(
                 'Bad projection specification, cannot include fields '
                 'or add computed fields during an exclusion projection: %s' % options)
-        if field == '_id':
-            continue
         if value in (0, 1, True, False):
-            filter_list.append(field)
+            if field != '_id':
+                filter_list.append(field)
             continue
         if not new_fields_collection:
             new_fields_collection = [{} for unused_doc in in_collection]
@@ -908,6 +942,14 @@ def _handle_count_stage(in_collection, database, options):
     return [{options: len(in_collection)}]
 
 
+def _handle_facet_stage(in_collection, database, options):
+    out_collection_by_pipeline = {}
+    for pipeline_title, pipeline in options.items():
+        out_collection_by_pipeline[pipeline_title] = list(process_pipeline(
+            in_collection, database, pipeline, None))
+    return [out_collection_by_pipeline]
+
+
 _PIPELINE_HANDLERS = {
     '$addFields': None,
     '$bucket': _handle_bucket_stage,
@@ -915,7 +957,7 @@ _PIPELINE_HANDLERS = {
     '$collStats': None,
     '$count': _handle_count_stage,
     '$currentOp': None,
-    '$facet': None,
+    '$facet': _handle_facet_stage,
     '$geoNear': None,
     '$graphLookup': _handle_graph_lookup_stage,
     '$group': _handle_group_stage,
